@@ -1,6 +1,4 @@
 """Variation fonts interpolation models."""
-from __future__ import print_function, division, absolute_import
-from fontTools.misc.py23 import *
 
 __all__ = ['nonNone', 'allNone', 'allEqual', 'allEqualTo', 'subList',
 	   'normalizeValue', 'normalizeLocation',
@@ -25,7 +23,10 @@ def allEqual(lst, mapper=None):
 	if not lst:
 		return True
 	it = iter(lst)
-	first = next(it)
+	try:
+		first = next(it)
+	except StopIteration:
+		return True
 	return allEqualTo(first, it, mapper=mapper)
 
 def subList(truth, lst):
@@ -136,7 +137,7 @@ def supportScalar(location, support, ot=True):
 			continue
 		if v <= lower or upper <= v:
 			scalar = 0.
-			break;
+			break
 		if v < peak:
 			scalar *= (v - lower) / (peak - lower)
 		else: # v > peak
@@ -189,19 +190,22 @@ class VariationModel(object):
 	  7: 0.6666666666666667}]
 	"""
 
-	def __init__(self, locations, axisOrder=[]):
+	def __init__(self, locations, axisOrder=None):
+		if len(set(tuple(sorted(l.items())) for l in locations)) != len(locations):
+			raise ValueError("locations must be unique")
+
 		self.origLocations = locations
-		self.axisOrder = axisOrder
+		self.axisOrder = axisOrder if axisOrder is not None else []
 
 		locations = [{k:v for k,v in loc.items() if v != 0.} for loc in locations]
-		keyFunc = self.getMasterLocationsSortKeyFunc(locations, axisOrder=axisOrder)
-		axisPoints = keyFunc.axisPoints
+		keyFunc = self.getMasterLocationsSortKeyFunc(locations, axisOrder=self.axisOrder)
 		self.locations = sorted(locations, key=keyFunc)
-		# TODO Assert that locations are unique.
-		self.mapping = [self.locations.index(l) for l in locations] # Mapping from user's master order to our master order
-		self.reverseMapping = [locations.index(l) for l in self.locations] # Reverse of above
 
-		self._computeMasterSupports(axisPoints, axisOrder)
+		# Mapping from user's master order to our master order
+		self.mapping = [self.locations.index(l) for l in locations]
+		self.reverseMapping = [locations.index(l) for l in self.locations]
+
+		self._computeMasterSupports(keyFunc.axisPoints)
 		self._subModels = {}
 
 	def getSubModel(self, items):
@@ -235,7 +239,11 @@ class VariationModel(object):
 				return -1 if v < 0 else +1 if v > 0 else 0
 			def key(loc):
 				rank = len(loc)
-				onPointAxes = [axis for axis,value in loc.items() if value in axisPoints[axis]]
+				onPointAxes = [
+					axis for axis, value in loc.items()
+					if axis in axisPoints
+					and value in axisPoints[axis]
+				]
 				orderedAxes = [axis for axis in axisOrder if axis in loc]
 				orderedAxes.extend([axis for axis in sorted(loc.keys()) if axis not in axisOrder])
 				return (
@@ -252,18 +260,6 @@ class VariationModel(object):
 		ret.axisPoints = axisPoints
 		return ret
 
-	@staticmethod
-	def lowerBound(value, lst):
-		if any(v < value for v in lst):
-			return max(v for v in lst if v < value)
-		else:
-			return value
-	@staticmethod
-	def upperBound(value, lst):
-		if any(v > value for v in lst):
-			return min(v for v in lst if v > value)
-		else:
-			return value
 	def reorderMasters(self, master_list, mapping):
 		# For changing the master data order without
 		# recomputing supports and deltaWeights.
@@ -276,7 +272,7 @@ class VariationModel(object):
 		self._subModels = {}
 		return new_list
 
-	def _computeMasterSupports(self, axisPoints, axisOrder):
+	def _computeMasterSupports(self, axisPoints):
 		supports = []
 		deltaWeights = []
 		locations = self.locations
@@ -377,7 +373,7 @@ class VariationModel(object):
 	def interpolateFromDeltasAndScalars(deltas, scalars):
 		v = None
 		assert len(deltas) == len(scalars)
-		for i,(delta,scalar) in enumerate(zip(deltas, scalars)):
+		for delta, scalar in zip(deltas, scalars):
 			if not scalar: continue
 			contribution = delta * scalar
 			if v is None:
@@ -444,6 +440,7 @@ def main(args):
 		pprint(locs)
 		doc.normalize()
 		print("Normalized locations:")
+		locs = [s.location for s in doc.sources]
 		pprint(locs)
 	else:
 		axes = [chr(c) for c in range(ord('A'), ord('Z')+1)]
