@@ -1,5 +1,3 @@
-from __future__ import (
-    print_function, division, absolute_import, unicode_literals)
 from collections import OrderedDict
 import fontTools.voltLib.ast as ast
 from fontTools.voltLib.lexer import Lexer
@@ -32,10 +30,19 @@ class Parser(object):
         self.lookups_ = SymbolTable()
         self.next_token_type_, self.next_token_ = (None, None)
         self.next_token_location_ = None
-        with open(path, "r") as f:
-            self.lexer_ = Lexer(f.read(), path)
+        self.make_lexer_(path)
         self.advance_lexer_()
 
+    def make_lexer_(self, file_or_path):
+        if hasattr(file_or_path, "read"):
+            filename = getattr(file_or_path, "name", None)
+            data = file_or_path.read()
+        else:
+            filename = file_or_path
+            with open(file_or_path, "r") as f:
+                data = f.read()
+        self.lexer_ = Lexer(data, filename)
+
     def parse(self):
         statements = self.doc_.statements
         while self.next_token_type_ is not None:
@@ -208,13 +215,16 @@ class Parser(object):
             if self.next_token_ == "MARK_GLYPH_SET":
                 self.advance_lexer_()
                 mark_glyph_set = self.expect_string_()
-            elif self.next_token_type_ == Lexer.STRING:
-                process_marks = self.expect_string_()
             elif self.next_token_ == "ALL":
                 self.advance_lexer_()
+            elif self.next_token_ == "NONE":
+                self.advance_lexer_()
+                process_marks = False
+            elif self.next_token_type_ == Lexer.STRING:
+                process_marks = self.expect_string_()
             else:
                 raise VoltLibError(
-                    "Expected ALL, MARK_GLYPH_SET or an ID. "
+                    "Expected ALL, NONE, MARK_GLYPH_SET or an ID. "
                     "Got %s" % (self.next_token_type_),
                     location)
         elif self.next_token_ == "SKIP_MARKS":
